__version__ = "0.1.0a2"

import inspect
import functools
import dis

def argument_names(func_=None, /, *, function = None):
    def decorator_argument_names(func):
        @functools.wraps(func)
        def wrapper_argument_names(*args, **kwargs):
            frame = inspect.currentframe().f_back
            try:
                instructions = list(dis.get_instructions(frame.f_code))
                relevant_instructions = [instr for instr in instructions if instr.offset <= frame.f_lasti]

                names = []

                for instr in reversed(relevant_instructions):
                    if instr.opname in ("LOAD_NAME",):
                        names.append(instr.argval)
                    if len(names) == len(args):
                        break
                
                if function is not None:
                    function(*names[::-1])

                return func(*args, **kwargs)
            finally:
                del frame
                return func(*args, **kwargs)
        return wrapper_argument_names
    
    if func_ is None:
        return decorator_argument_names
    else:
        return decorator_argument_names(func_)


from urllib.request import urlopen
from urllib.error import URLError
import json

from packaging import version

from . import __version__ as current_version

def main():
    pypi_url = "https://pypi.python.org/pypi/argument-names/json"
    try:
        response = urlopen(pypi_url, timeout=1)

        data = json.loads(response.read())
        
        latest_version = data["info"]["version"]
        package_name = data["info"]["name"]
    except URLError:
        print("Network issue: Unable to check for updates.")
    except (KeyError, ValueError):
        print("Error: Unexpected data format from PyPI.")
    else:
        if version.parse(current_version) < version.parse(latest_version):
            print(f"A new version of {package_name} is available: {latest_version} (Current version: {current_version})")
        else:
            print(f"{package_name} is already up to date (version {current_version}).")
        return
    
    print(f"argument-names {current_version}")